--[[
Premo Power version 2.0.1.0 release
Unit Power movable bar for Paladins, Priests, Monks, Rogues and Warlocks
By Azmaedus 
6.0.3 compatible
License GPLv2

Change log
-----------------------
01/25/2015 - 2.0.0.0 Complete re-write, compatible 6.0.3
01/26/2015 - 2.0.0.1 Fixed an issue with a missing localization
01/27/2015 - 2.0.0.2 Updated LibFrameX-1.0, converted PremoPower to use new structure
01/30/2015 - 2.0.0.3 Fixed a few issues with display/update of the frames
01/31/2015 - 2.0.0.4 Updated to use the new and improved LibFrameX library
02/17/2015 - 2.0.0.5 Fixed issue with inverted check between disc & shadow (priest) stacks
02/24/2015 - 2.0.0.6 Fixed issue with switching profiles and updated toc for 6.1
02/26/2015 - 2.0.0.7 Fixed an issue where changing specs would incorrectly display the number of max chi (from 4 to 6)
--]]

-- Lua APIs
local tinsert, tconcat, tremove = table.insert, table.concat, table.remove
local fmt, tostring = string.format, tostring
local select, pairs, next, type, unpack = select, pairs, next, type, unpack
local loadstring, assert, error = loadstring, assert, error
local setmetatable, getmetatable, rawset, rawget = setmetatable, getmetatable, rawset, rawget

-- Load the Ace libraries --
PremoPower = LibStub("AceAddon-3.0"):NewAddon("PremoPower", "AceConsole-3.0", "AceEvent-3.0")
local L = LibStub("AceLocale-3.0"):GetLocale("PremoPower")

local LFX = LibStub("LibFrameX-1.0")
assert(LFX, "LibFrameX-1.0 not loaded")

local LTC = LibStub("LibTColor-1.0")
assert(LTC, "LibTColor-1.0 not loaded")

-- Embedding functions from LFX
PremoPower.makeFrames = LFX.makeFrames
PremoPower.getBD = LFX.getBD
PremoPower.setBD = LFX.setBD
PremoPower.getBRGBA = LFX.getBRGBA
PremoPower.setBRGBA = LFX.setBRGBA
PremoPower.getRGBA = LFX.getRGBA
PremoPower.setRGBA = LFX.setRGBA
PremoPower.getDimensions = LFX.getDimensions
PremoPower.setDimensions = LFX.setDimensions
PremoPower.getVisibility = LFX.getVisibility
PremoPower.setVisibility = LFX.setVisibility
PremoPower.getPosition = LFX.getPosition
PremoPower.setPosition = LFX.setPosition
PremoPower.getFrameCaption = LFX.getFrameCaption
PremoPower.setFrameCaption = LFX.setFrameCaption

-- ---------------------------------------------
-- PremoPower globals
-- ---------------------------------------------
PremoPower.gVersion = GetAddOnMetadata("PremoPower", "Version")
PremoPower.gVersionID = {2,0,0,6,2} -- release major, release minor, build major, build minor, type (alpha 0, beta 1, release 2)
PremoPower.gCurReleaseDate = "02/24/2015"
_G["PremoPower"] = PremoPower -- let the world see PremoPower

local CLASS_PALADIN = "PALADIN"
local CLASS_MONK = "MONK"
local CLASS_ROGUE = "ROGUE"
local CLASS_PRIEST = "PRIEST"
local CLASS_WARLOCK = "WARLOCK"

local MAX_BLIP = 6

-- Class specific
local MAX_PALADIN_BLIP = 5
local MAX_PRIEST_DISC_BLIP = 5
local MAX_ROGUE_BLIP = 5
local MAX_WARLOCK_DESTRO_BLIP = 5

-- Specialization specific
local SPEC_PRIEST_DISC = 2
local SPEC_PRIEST_SHADOW = 3
local SPEC_WARLOCK_DESTRO = 3

-- Table for [Load On Demand] module options registration
local moduleOptions = {} 

-- ---------------------------------------------
-- create minimap icon
-- ---------------------------------------------
function PremoPower:CreateMinimapIcon()
	local plugin = LibStub("LibDataBroker-1.1"):NewDataObject("PremoPower", {
		type = "data source",
		text = "0",
		icon = "Interface\\AddOns\\PremoPower\\media\\icon",
	})

	function plugin.OnClick(self, button)
		if button == "LeftButton" then
			PremoPower:ShowConfig()
		elseif button == "RightButton" then
			if PremoPower.db.profile.frame.hideframes then
				PremoPower.db.profile.frame.hideframes = false
				PremoPower:setVisibility("PPBG", true)
				PremoPower:OnUpdateFrames()
			else
				PremoPower.db.profile.frame.hideframes = true
				PremoPower:setVisibility("PPBG", false)
				PremoPower:OnUpdateFrames()
			end
		end
	end

	local hint = L["ICON_CLICK"]
	local hint2 = L["ICON_RCLICK"]
	function plugin.OnTooltipShow(tt)
		tt:AddLine(L["ADDON_NAME"])
		tt:AddLine(" ")
		tt:AddLine(hint, 0.2, 1, 0.2, 1)
		tt:AddLine(" ")
		tt:AddLine(hint2, 0.2, 1, 0.2, 1)
	end

	local ldbif = CreateFrame("Frame")
	ldbif:SetScript("OnEvent", function()
		local icon = LibStub("LibDBIcon-1.0", true)
		icon:Register("PremoPower", plugin, PremoPower.db.profile.minimap)
	end)
	ldbif:RegisterEvent("PLAYER_LOGIN")
end

-- ---------------------------------------------
-- Create the default settings for this addon 
-- ---------------------------------------------
local defaults = { 
	profile =  {
		frame = {
			saved = true,
			pos = {
				x = 0,
				y = 0,
				scale = 1,
				a = "TOPLEFT",
				r = UIParent,
				rp = "CENTER",
			},
			dim = {
				w = 20,
				h = 20,
				s = 1,
			},
			coloron = {
				r = 246,
				g = 227,
				b = 19,
				a = 0.9
			},
			coloroff = {
				r = 246,
				g = 227,
				b = 19,
				a = 0.3
			},
			hidetext = false,
			hideframes = false,
		},
		minimap = {
			hide = false,
		},
	},
}

-- LFX table
local pplfxt = {
	{
		root = true, -- creates the frame but doesn't draw any backdrop
		ftype = "Frame",
		name = "PPBG",
		rgba = {r = 0, g = 0, b = 0, a = 1},
		brgba = {r = 0, g = 0, b = 0, a = 1},
		dim = {w = 200, h = 27, s = 1},
		pos = {x = 0, y = 0, a = "TOPLEFT", r = UIParent, rp = "CENTER"},
		show = true,
		movable = {
			enabled = true, 
			drag = {"RightButton"},
		},
		scripts = {
			{
				event = "OnDragStart", 
				func = function( self )
					if IsAltKeyDown() then
						self:StartMoving()
					end
				end,
			},
			{
				event = "OnDragStop", 
				func = function( self )
					self:StopMovingOrSizing()
					local pos = PremoPower.db.profile.frame.pos
					pos.a, pos.r, pos.rp, pos.x, pos.y = self:GetPoint(1)
				end,
			},
		},
		children = {
			{
				ftype = "Frame",
				name = "PPFrame1",
				pos = {x = 0, y = 1, a = "BOTTOMLEFT", r = "PPBG", rp = "BOTTOMLEFT"},
				bd = { bgFile = [[Interface\ChatFrame\ChatFrameBackground]], edgeFile = [[Interface/Tooltips/UI-Tooltip-Border]], tile = true, tileSize = 4, edgeSize = 4, insets = { left = 1, right = 1, top = 1, bottom = 1 }},
				rgba = {r = 246, g = 227, b = 19, a = 0.9},
				brgba = {r = 0, g = 0, b = 0, a = 0.7},
				dim = {w = 25, h = 25, s = 1},
				text = {	show = true, caption = "", color = {r = 255, g = 255,	b = 255,	a = 1}, fs = "NumberFont_Shadow_Small", halign = "CENTER", valign = "CENTER"},
				show = true,
			},
			{
				ftype = "Frame",
				name = "PPFrame2",
				pos = {x = 2, y = 0, a = "BOTTOMLEFT", r = "PPFrame1", rp = "BOTTOMRIGHT"},
				bd = { bgFile = [[Interface\ChatFrame\ChatFrameBackground]], edgeFile = [[Interface/Tooltips/UI-Tooltip-Border]], tile = true, tileSize = 4, edgeSize = 4, insets = { left = 1, right = 1, top = 1, bottom = 1 }},
				rgba = {r = 246, g = 227, b = 19, a = 0.9},
				brgba = {r = 0, g = 0, b = 0, a = 0.7},
				dim = {w = 25, h = 25, s = 1},
				text = {	show = true, caption = "", color = {r = 255, g = 255,	b = 255,	a = 1}, fs = "NumberFont_Shadow_Small", halign = "CENTER", valign = "CENTER"},
				show = true,
			},
			{
				ftype = "Frame",
				name = "PPFrame3",
				pos = {x = 2, y = 0, a = "BOTTOMLEFT", r = "PPFrame2", rp = "BOTTOMRIGHT"},
				bd = { bgFile = [[Interface\ChatFrame\ChatFrameBackground]], edgeFile = [[Interface/Tooltips/UI-Tooltip-Border]], tile = true, tileSize = 4, edgeSize = 4, insets = { left = 1, right = 1, top = 1, bottom = 1 }},
				rgba = {r = 246, g = 227, b = 19, a = 0.9},
				brgba = {r = 0, g = 0, b = 0, a = 0.7},
				dim = {w = 25, h = 25, s = 1},
				text = {	show = true, caption = "", color = {r = 255, g = 255,	b = 255,	a = 1}, fs = "NumberFont_Shadow_Small", halign = "CENTER", valign = "CENTER"},
				show = true,
			},
			{
				ftype = "Frame",
				name = "PPFrame4",
				pos = {x = 2, y = 0, a = "BOTTOMLEFT", r = "PPFrame3", rp = "BOTTOMRIGHT"},
				bd = { bgFile = [[Interface\ChatFrame\ChatFrameBackground]], edgeFile = [[Interface/Tooltips/UI-Tooltip-Border]], tile = true, tileSize = 4, edgeSize = 4, insets = { left = 1, right = 1, top = 1, bottom = 1 }},
				rgba = {r = 246, g = 227, b = 19, a = 0.9},
				brgba = {r = 0, g = 0, b = 0, a = 0.7},
				dim = {w = 25, h = 25, s = 1},
				text = {	show = true, caption = "", color = {r = 255, g = 255,	b = 255,	a = 1}, fs = "NumberFont_Shadow_Small", halign = "CENTER", valign = "CENTER"},
				show = true,
			},
			{
				ftype = "Frame",
				name = "PPFrame5",
				pos = {x = 2, y = 0, a = "BOTTOMLEFT", r = "PPFrame4", rp = "BOTTOMRIGHT"},
				bd = { bgFile = [[Interface\ChatFrame\ChatFrameBackground]], edgeFile = [[Interface/Tooltips/UI-Tooltip-Border]], tile = true, tileSize = 4, edgeSize = 4, insets = { left = 1, right = 1, top = 1, bottom = 1 }},
				rgba = {r = 246, g = 227, b = 19, a = 0.9},
				brgba = {r = 0, g = 0, b = 0, a = 0.7},
				dim = {w = 25, h = 25, s = 1},
				text = {	show = true, caption = "", color = {r = 255, g = 255,	b = 255,	a = 1}, fs = "NumberFont_Shadow_Small", halign = "CENTER", valign = "CENTER"},
				show = true,
			},
			{
				ftype = "Frame",
				name = "PPFrame6",
				pos = {x = 2, y = 0, a = "BOTTOMLEFT", r = "PPFrame5", rp = "BOTTOMRIGHT"},
				bd = { bgFile = [[Interface\ChatFrame\ChatFrameBackground]], edgeFile = [[Interface/Tooltips/UI-Tooltip-Border]], tile = true, tileSize = 4, edgeSize = 4, insets = { left = 1, right = 1, top = 1, bottom = 1 }},
				rgba = {r = 246, g = 227, b = 19, a = 0.9},
				brgba = {r = 0, g = 0, b = 0, a = 0.7},
				dim = {w = 25, h = 25, s = 1},
				text = {	show = true, caption = "", color = {r = 255, g = 255,	b = 255,	a = 1}, fs = "NumberFont_Shadow_Small", halign = "CENTER", valign = "CENTER"},
				show = true,
			},
		},
	},
}

--[[=============================================================
						OnInitialize
    =============================================================]]
function PremoPower:OnInitialize()
	self.db = LibStub("AceDB-3.0"):New("PremoPowerDB", defaults, true)
	self.db.RegisterCallback(self, "OnProfileChanged", "OnProfileChanged")
	self.db.RegisterCallback(self, "OnProfileCopied", "OnProfileChanged")
	self.db.RegisterCallback(self, "OnProfileReset", "OnProfileChanged")

	LibStub("AceConfigRegistry-3.0"):RegisterOptionsTable("PremoPowerDB", self.GenerateOptions)
	LibStub("AceConfig-3.0"):RegisterOptionsTable("PremoPower", self.slashcmd, "ppower")
	
	local ACD3 = LibStub("AceConfigDialog-3.0")
	self.optionsFrames = {}
	self.optionsFrames.PremoPower = ACD3:AddToBlizOptions("PremoPowerDB", L["ADDON_NAME"], nil)
	self:RegisterModuleOptions("PPSlashCmd", self.slashcmd, L["ADDON_SLASHCMD"])
	self:RegisterModuleOptions("Profiles", function() return LibStub("AceDBOptions-3.0"):GetOptionsTable(self.db) end, L["Profiles"])
	self.optionsFrames.About = ACD3:AddToBlizOptions("PremoPowerDB", L["OPT_ABOUT"], L["ADDON_NAME"], "About")

	-- character information
	self.gPlayerInfo = {
	  _, _, class = string.upper(UnitClass( 'player' )),
	  level = UnitLevel("player"),
	  spec = GetSpecialization(),
	}

	-- check for loading default colors
	local con = self.db.profile.frame.coloron
	local coff = self.db.profile.frame.coloroff
	if not con and coff then
		local c = self.gPlayerInfo.class
		local s = self.gPlayerInfo.spec
		if c == CLASS_PALADIN then
			con = {r=245,g=140,b=186,a=1}
			coff = {r=245,g=140,b=186,a=0.3}
		elseif c == CLASS_MONK then
			con = {r=0,g=255,b=150,a=1}
			coff = {r=0,g=255,b=150,a=0.3}
		elseif c == CLASS_ROGUE then
			con = {r=255,g=245,b=105,a=1}
			coff = {r=255,g=245,b=105,a=0.3}
		elseif (c == CLASS_PRIEST) and (s == SPEC_PRIEST_DISC or s == SPEC_PRIEST_SHADOW) then
			con = {r=255,g=255,b=255,a=1}
			coff = {r=255,g=255,b=255,a=0.3}
		elseif c == CLASS_WARLOCK and s == SPEC_WARLOCK_DESTRO then
			con = {r=148,g=130,b=201,a=1}
			coff = {r=148,g=130,b=201,a=0.3}
		end
	end
	
	self:makeFrames(pplfxt)
	self:CreateMinimapIcon()
	
	if self.db.profile.frame.hideframes then
		self:setVisibility("PPBG", false)
	else
		self:setVisibility("PPBG", true)
	end
	
	self:OnUpdateFrames()

	self:Print(L["ADDON_NAME"] .. " " .. self.gVersion .. " " .. L["ADDON_LOADED"])

	self.OnInitialize = nil
end

-- Register module options
function PremoPower:RegisterModuleOptions(name, optionTbl, displayName)
	if moduleOptions then
		moduleOptions[name] = optionTbl
	else
		self.options.args[name] = (type(optionTbl) == "function") and optionTbl() or optionTbl
	end
	self.optionsFrames[name] = LibStub("AceConfigDialog-3.0"):AddToBlizOptions("PremoPowerDB", displayName, L["ADDON_NAME"], name)
end

function PremoPower.GenerateOptions()
	if not PremoPower.options then
		PremoPower.GenerateOptionsInternal()
		PremoPower.GenerateOptionsInternal = nil
		moduleOptions = nil
	end
	return PremoPower.options
end

-- Option table for the slash command only
PremoPower.slashcmd = {
	type = "group",
	name = L["ADDON_SLASHCMD"],
	order = -3,
	func = function() PremoPower:ShowConfig() end,
	args = {
		intro = {
			order = 1,
			type = "description",
			name = L["ADDON_SLASHCMD_DESC"],
			cmdHidden = true,
		},
		show = {
			order = 4,
			type = "execute",
			name = L["CMD_SHOW"],
			desc = L["CMD_SHOW_DESC"],
			func = function()
				PremoPower:setVisibility("PPBG", true)
				PremoPower:OnUpdateFrames()
			end,
		},
		hide = {
			order = 5,
			type = "execute",
			name = L["CMD_HIDE"],
			desc = L["CMD_HIDE_DESC"],
			func = function()
				PremoPower:setVisibility("PPBG", false)
				PremoPower:OnUpdateFrames()
			end,
		},
		showicon = {
			order = 6,
			type = "execute",
			name = L["CMD_MINIMAP_SHOW"],
			desc = L["CMD_MINIMAP_SHOW_DESC"],
			func = function()
				local icon = LibStub("LibDBIcon-1.0", true)
				PremoPower.db.profile.minimap.hide = false
				icon:Show("PremoPower")
			end,
		},
		hideicon = {
			order = 7,
			type = "execute",
			name = L["CMD_MINIMAP_HIDE"],
			desc = L["CMD_MINIMAP_HIDE_DESC"],
			func = function()
				local icon = LibStub("LibDBIcon-1.0", true)
				PremoPower.db.profile.minimap.hide = true 
				icon:Hide("PremoPower")
			end,
		},
		config = {
			type = "execute",
			name = L["CMD_CONFIG"],
			desc = L["CMD_CONFIG_DESC"],
			func = function() PremoPower:ShowConfig() end,
			guiHidden = true,
		},
	},
}

function PremoPower.GenerateOptionsInternal()
	-- Create the options variable to use in the config screen --
	PremoPower.options = {
		name = L["OPT_PREMOPOWER_OPTIONS"],
		handler = PremoPower,
		type = "group",
		childGroups  = "tab",
		args = {
			PremoPower = {
				name = L["ADDON_NAME"],
				desc = L["OPT_PP_DESC"],
				type = "group",
				order = 1,
				args = {
					hdrcolor = {
						name = L["HDR_COLOR"],
						type = "header",
						order = 1,
					},
					coloron = {
						type = "color",
						name = L["COLOR_ON"],
						hasAlpha = true,
						get = function(info)
							local c = PremoPower.db.profile.frame.coloron
							return c.r/256, c.g/256, c.b/256, c.a
						end,
						set = function(info, r, g, b, a)
							local c = PremoPower.db.profile.frame.coloron
							c.r, c.g, c.b, c.a = r*256, g*256, b*256, a
							PremoPower:OnUpdateFrames()
						end,
						order = 2,
						width = "full",
					},
					coloroff = {
						type = "color",
						name = L["COLOR_OFF"],
						hasAlpha = true,
						get = function(info)
							local c = PremoPower.db.profile.frame.coloroff
							return c.r/256, c.g/256, c.b/256, c.a
						end,
						set = function(info, r, g, b, a)
							local c = PremoPower.db.profile.frame.coloroff
							c.r, c.g, c.b, c.a = r*256, g*256, b*256, a
							PremoPower:OnUpdateFrames()
						end,
						order = 3,
						width = "full",
					},
					hdrdim = {
						name = L["HDR_DIM"],
						type = "header",
						order = 10,
					},
					width = {
						name = L["INP_WIDTH"],
						type = "input",
						width = "half",
						set = function(info, value)	
							dim = PremoPower.db.profile.frame.dim
							dim.w = value 
							PremoPower:OnUpdateFrames()
						end,
						get = function(info) return PremoPower.db.profile.frame.dim.w end,
						order = 11,
					},
					height = {
						name = L["INP_HEIGHT"],
						type = "input",
						width = "half",
						set = function(info, value)	
							dim = PremoPower.db.profile.frame.dim
							dim.h = value 
							PremoPower:OnUpdateFrames()
						end,
						get = function(info) return PremoPower.db.profile.frame.dim.h end,
						order = 14,
					},
					scale = {
						name = L["INP_SCALE"],
						type = "input",
						width = "half",
						set = function(info, value)
							dim = PremoPower.db.profile.frame.dim
							dim.s = value 
						end,
						get = function(info) return PremoPower.db.profile.frame.dim.s end,
						order = 17,
					},
					hdrdim = {
						name = "",
						type = "header",
						order = 18,
					},
					 hidetext = {
						type = "toggle",
						name = L["TOG_HIDETEXT"],
						desc = L["TOG_HIDETEXT_DESC"],
						set = function(info, value)
							PremoPower.db.profile.frame.hidetext = value
							PremoPower:OnUpdateFrames()
						end,
						get = function(info) return PremoPower.db.profile.frame.hidetext end,
						width = "normal",
						order = 20,
					},
					 hideicon = {
						type = "toggle",
						name = L["TOG_ICON"],
						desc = L["TOG_ICON_DESC"],
						set = function(info, value)
							local icon = LibStub("LibDBIcon-1.0", true)
							PremoPower.db.profile.minimap.hide = value
							if PremoPower.db.profile.minimap.hide then
								icon:Hide("PremoPower")
							else
								icon:Show("PremoPower")
							end
						end,
						get = function(info) return PremoPower.db.profile.minimap.hide end,
						width = "normal",
						order = 21,
					},
				},
			},
			About = {
				name = L["OPT_ABOUT"],
				desc = "",
				type = "group",
				order = 3,
				args = {
					desctitle = {
						name = LTC:setColor( L["ADDON_NAME"] .. " " .. PremoPower.gVersion .. " - " .. PremoPower.gCurReleaseDate, LTC_GREEN ),
						type = "description",
						width = "full",
						fontSize = "medium",
						order = 1,
					},
					descauthor = {
						name = LTC:setColor(L["ADDON_AUTHOR"] .. " " .. "Azmaedus", LTC_BLUE),
						type = "description",
						width = "full",
						fontSize = "medium",
						order = 2,
					},
					descslashhdr = {
						name = "\r" .. L["ABOUT_SLASHCMD_TITLE"],
						type = "description",
						width = "full",
						fontSize = "medium",
						order = 3,
					},
					descslashplay = {
						name = L["ABOUT_SLASHCMD_CONFIG"],
						type = "description",
						width = "full",
						fontSize = "small",
						order = 4,
					},
				},
			},
		},
	}

	for k, v in pairs(moduleOptions) do
		PremoPower.options.args[k] = (type(v) == "function") and v() or v
	end

	-- Add ordering data to the option table generated by AceDBOptions-3.0
	PremoPower.options.args.Profiles.order = -2
end

function PremoPower:ShowConfig()
	-- Open the profiles tab before, so the menu expands
	InterfaceOptionsFrame_OpenToCategory(self.optionsFrames.Profiles)
	InterfaceOptionsFrame_OpenToCategory(self.optionsFrames.PremoPower)
end

--[[=============================================================
						OnEnable
    =============================================================]]
function PremoPower:OnEnable()
	self:RegisterEvent("UNIT_COMBO_POINTS", "OnUpdateFrames")
	self:RegisterEvent("PLAYER_TARGET_CHANGED", "OnUpdateFrames")
	self:RegisterEvent("UNIT_POWER", "OnUpdateFrames")
	self:RegisterEvent("UNIT_AURA", "OnUpdateFrames")
	self:RegisterEvent("ACTIVE_TALENT_GROUP_CHANGED", "OnUpdateFrames")
end

--[[=============================================================
						OnDisable
    =============================================================]]
function PremoPower:OnDisable()
	self:UnregisterEvent("UNIT_COMBO_POINTS")
	self:UnregisterEvent("PLAYER_TARGET_CHANGED")
	self:UnregisterEvent("UNIT_POWER")
	self:UnregisterEvent("UNIT_AURA")
	self:UnregisterEvent("ACTIVE_TALENT_GROUP_CHANGED")
end

function PremoPower:UpdateComponents()
	local pos = self.db.profile.frame.pos
	self:setPosition("PPBG", pos)

	if self.db.profile.hideframes == true then
		self:setVisibility("PPBG", false)
	else
		self:setVisibility("PPBG", true)
	end

	local icon = LibStub("LibDBIcon-1.0", true)
	if self.db.profile.minimap.hide then
		icon:Hide("PremoPower")
	else
		icon:Show("PremoPower")
	end
end

-- -------------------------------------------
-- Events
-- -------------------------------------------
function PremoPower:OnProfileChanged(event, database, newProfileKey)
	local db = database.profile
	self.db.profile = db
	self.db.profile.frame = db.frame
	self.db.profile.minimap = db.minimap

	--self:UpdateComponents()
	self:OnUpdateFrames()
end
	
-- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

function PremoPower:OnUpdateFrames()
	local c = self.gPlayerInfo.class 
	local s = PremoPower.gPlayerInfo.spec
	if c == CLASS_ROGUE then
		self:updateFrames( MAX_ROGUE_BLIP, GetComboPoints("player" ,"target"))
	elseif c == CLASS_MONK then
		self:updateFrames( UnitPowerMax("player", SPELL_POWER_CHI), UnitPower("player", SPELL_POWER_CHI))
	elseif c == CLASS_PALADIN then
		self:updateFrames( UnitPowerMax("player", SPELL_POWER_HOLY_POWER), UnitPower("player", SPELL_POWER_HOLY_POWER))
	elseif c == CLASS_PRIEST then
		if s == SPEC_PRIEST_DISC then -- disc
			self:updateFrames( MAX_PRIEST_DISC_BLIP, self:getEvangelismStacks())
		elseif s == SPEC_PRIEST_SHADOW then -- shadow
			self:updateFrames( UnitPowerMax("player", SPELL_POWER_SHADOW_ORBS), UnitPower( "player", SPELL_POWER_SHADOW_ORBS ))
		end
	elseif c == CLASS_WARLOCK then
		if s == SPEC_WARLOCK_DESTRO then
			self:updateFrames( MAX_WARLOCK_DESTRO_BLIP, self:getBurningEmberStacks()) 
		end
	end
end

function PremoPower:getEvangelismStacks()
	local _, _, _, count = UnitAura( "player", "Evangelism" ) --81661
	if count == nil then
		return 0
	else
		return count
	end
end

function PremoPower:getBurningEmberStacks()
	local _, _, _, count = UnitAura( "player", "Burning Embers" ) --108647
	if count == nil then
		return 0
	else
		return count
	end
end

-- Function to update frames in regards to # of frames/state required
-- total: maximum frames to display
-- [cur]: Number of frames active
function PremoPower:updateFrames(total, cur)
	-- loop through the lfx table's children
	local rgba = {}
	for i = 1, MAX_BLIP do
		-- on/off rgba and dimsensions
		if i <= cur then
			rgba = self.db.profile.frame.coloron
		elseif i > cur and i <= total then
			rgba = self.db.profile.frame.coloroff
		end

		-- set text
		if self.db.profile.frame.hidetext == true then
			PremoPower:setFrameCaption("PPFrame" .. i, "")
		else
			PremoPower:setFrameCaption("PPFrame".. i, tostring(i))
		end

		PremoPower:setRGBA("PPFrame" .. i, rgba)
		PremoPower:setDimensions("PPFrame" .. i, self.db.profile.frame.dim)
		
		if i > total then
			PremoPower:setVisibility("PPFrame" .. i, false) -- name, visibility, affect children = false
		else
			PremoPower:setVisibility("PPFrame" .. i, true) -- name, visibility, affect children = false
		end
	end
end
